#!/usr/bin/env perl

use XyceRegression::Tools;
$Tools = XyceRegression::Tools->new();

# The input arguments to this script are:
# $ARGV[0] = location of Xyce binary
# $ARGV[1] = location of xyce_verify.pl script
# $ARGV[2] = location of compare script 
# $ARGV[3] = location of circuit file to test
# $ARGV[4] = location of gold standard prn file

# If Xyce does not produce the appropriate output files, then we return exit code 14.
# If Xyce succeeds, but the test fails, then we return exit code 2.
# If the shell script fails for some reason, then we return exit code 1.

# Since the shell script runs Xyce and the comparison program, it is
# responsible for capturing any error output from Xyce and the STDOUT & STDERR
# from the comparison program.  The outside script, run_xyce_regression,
# expects the STDERR output from Xyce to go into $CIRFILE.err, the STDOUT
# output from comparison to go into $CIRFILE.prn.out and the STDERR output from
# comparison to go into $CIRFILE.prn.err.

use Getopt::Long;
&GetOptions( "verbose!" => \$verbose );

$XYCE=$ARGV[0];
$XYCE_VERIFY=$ARGV[1];
$XYCE_COMPARE=$ARGV[2];
$CIRFILE=$ARGV[3];
$GOLDPRN=$ARGV[4];

$GOLDPRN =~ s/\.prn$//; # remove the .prn at the end.

`rm -f $CIRFILE.TRADJ.prn $CIRFILE.out $CIRFILE.err`;
`rm -f $CIRFILE.errmsg.out $CIRFILE.errmsg.err`;

# run Xyce
$CMD="$XYCE $CIRFILE > $CIRFILE.out 2> $CIRFILE.err";
$retval = system("$CMD");
if ($retval != 0) 
{
  if ($retval & 127)
  {
    print "Exit code = 13\n"; 
    printf STDERR "Xyce crashed with signal %d on file %s\n",($retval&127),$CIRFILE; 
    exit 13;
  }
  else
  {
    print "Exit code = 10\n"; 
    printf STDERR "Xyce exited with exit code %d on %s\n",$retval>>8,$CIRFILE; 
    exit 10;
  }
}

if ( not -s "$CIRFILE.TRADJ.prn") 
{
  print "Missing output file $CIRFILE.TRADJ.prn\n";
  print "Exit code =14\n";
  exit 14;
}

# these search strings are supposed to occur one right after the other in the
# error output.
@searchstrings = ("Transient adjoint output cannot be written in PROBE or RAW",
                  "format, using standard format instead",
);
$retval = $Tools->checkError("$CIRFILE.out",@searchstrings);
if ($retval != 0)
{
  print "Check on warning message failed\n";
  print "Exit code = $retval\n"; exit $retval; 
} 

# now compare the test and gold TRADJ.prn files
$absTol=1e-5;
$relTol=1e-3;
$zeroTol=1e-10;
$fc = $XYCE_VERIFY;
$fc=~ s/xyce_verify/file_compare/;
$CMD="$fc $CIRFILE.TRADJ.prn $GOLDPRN.TRADJ.prn $absTol $relTol $zeroTol > $CIRFILE.errmsg.out 2> $CIRFILE.errmsg.err";
$retval = system("$CMD");
# check the return value
if ( $retval != 0 )
{
  print "test failed comparison of TRADJ.prn file vs. gold TRADJ.prn file!\n";
  print "Exit code = 2\n";
  exit 2;
}
else
{
  print "test passed comparison of TRADJ.prn file vs. gold TRADJ.prn file!\n";
  print "Exit code = 0\n";
  exit 0;
}

